// TODO : Reoccuring investments

use based::get_pg;
use rocket::{form::Form, post, response::Redirect, FromForm};
use sqlx::prelude::FromRow;

use crate::PathSegment;

#[derive(Debug, Clone, FromRow)]
pub struct Investment {
    pub id: i64,
    pub project: String,
    pub title: String,
    pub description: String,
    pub amount: f64,
    pub when: String,
    pub payed: bool,
}

impl Investment {
    pub async fn new_investment(
        project: &str,
        amount: f64,
        title: &str,
        description: &str,
    ) -> Investment {
        let project = project.trim_end_matches("/");
        sqlx::query_as(
            "INSERT INTO investment (project, title, description, amount) 
             VALUES ($1, $2, $3, $4) 
             RETURNING *",
        )
        .bind(project)
        .bind(title)
        .bind(description)
        .bind(amount)
        .fetch_one(get_pg!())
        .await
        .unwrap()
    }

    pub async fn pay(id: i64) -> String {
        let res: (String,) =
            sqlx::query_as("UPDATE investment SET payed = true WHERE id = $1 RETURNING project")
                .bind(id)
                .fetch_one(get_pg!())
                .await
                .unwrap();
        res.0
    }

    pub async fn get_all_investments_of(prj: &str) -> Vec<Investment> {
        let prj = prj.trim_end_matches("/");
        sqlx::query_as("SELECT * FROM investment WHERE project = $1 ORDER BY \"when\" DESC")
            .bind(prj)
            .fetch_all(get_pg!())
            .await
            .unwrap()
    }

    pub async fn get_total_of(prj: &str) -> (f64, f64) {
        let prj = prj.trim_end_matches("/");
        let payed: Result<(f64,), sqlx::Error> = sqlx::query_as(
            "SELECT SUM(amount) FROM investment WHERE project = $1 AND payed = true",
        )
        .bind(prj)
        .fetch_one(get_pg!())
        .await;
        let total: Result<(f64,), sqlx::Error> =
            sqlx::query_as("SELECT SUM(amount) FROM investment WHERE project = $1")
                .bind(prj)
                .fetch_one(get_pg!())
                .await;
        if let (Ok(payed), Ok(total)) = (payed, total) {
            (payed.0, total.0)
        } else {
            (0.00, 0.00)
        }
    }
}

#[derive(FromForm)]
pub struct NewInvestForm {
    title: String,
    amount: String,
    description: String,
}

#[post("/invest/<path..>", data = "<form>")]
pub async fn post_new_invest(path: PathSegment, form: Form<NewInvestForm>) -> Redirect {
    let root = path.to_str();
    Investment::new_investment(
        &root,
        form.amount.parse().unwrap(),
        &form.title,
        &form.description,
    )
    .await;
    Redirect::to(format!("/prj/{}?tab=invest", root))
}
